@@ -5,6 +5,7 @@ use warnings;
 
 use Fcntl;
 use Digest::SHA qw(sha1);
+use POSIX qw(ceil floor);
 
 use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS $VERSION);
 
@@ -12,7 +13,7 @@ require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(random_uuid_160);
-$VERSION = '0.03';
+$VERSION = '0.04';
 
 #
 # Random number generator
@@ -81,15 +82,18 @@ $VERSION = '0.03';
 # However, I believe that the benefits of using a readily-available
 # implementation (along with the other advantages listed) outweigh
 # this minor disadvantage. For those reasons, I will use SHA-1 here.
-#
+
+
+# RNG object only contains seed and current values so store them in an
+# array rather than a hash for efficiency.
+
+use constant CURRENT => 0;
+use constant SEED    => 1;
 
 sub new {
 
   my ($class, $seed) = @_;
-  my $self = {
-	      seed => $seed,
-	      current => undef,
-	     };
+  my $self = [ undef, $seed ];
 
   bless $self, $class;
   $self->seed($seed);
@@ -99,10 +103,7 @@ sub new {
 sub new_random {
 
   my $class = shift;
-  my $self = {
-	      seed => undef,
-	      current => undef,
-	     };
+  my $self = [ undef, undef ];
 
   bless $self, $class;
   $self->seed_random();
@@ -121,8 +122,8 @@ sub seed {
   die "seed: self object not a reference\n" unless ref($self);
 
   $seed = "\0" x 20 unless defined($seed);
-  $self->{seed} = $seed;
-  $self->{current} = $seed;
+  $self->[SEED] = $seed;
+  $self->[CURRENT] = $seed;
   return $seed;
 }
 
@@ -134,63 +135,66 @@ sub seed_random {
 }
 
 sub get_seed {
-  return  shift->{seed};
+  return  shift->[SEED];
 }
 
 # As per Perl's rand, return a float value, 0 <= value < x
 sub rand {
   my ($self,$max) = @_;
-  $max = $max || 1;
-  $max = abs($max);		# don't allow negative values
-
+  $max = $max || 1.0;
   $max += 0.0;			# ensure max is a float
 
+  my ($maxint,$r, $ratio, $current) = (0xffffffff);
   while(1) {
-    $self->{current} = sha1($self->{current}); # advance to next rand
+    # advance to the next rand
+    $current = $self->[CURRENT] = sha1($self->[CURRENT]);
 
-    # unpack 5 32-bit words from the 160-bit SHA sum
-    my @uints = unpack "N5", $self->{current};
+    # Unpack first 32-bit little-endian word from SHA1 value
+    $r = unpack "V1", $current;
 
     # We calculate the rand by max * uint/(max 32-bit int).
-    while (@uints>=1) {
-      my $r = shift @uints;
-      #      $r ^= shift @uints;
-      #      $r ^= shift @uints;
-      #      $r ^= shift @uints;
-      #      $r ^= shift @uints;
-      my $maxint = 2**32 - 1;
-      my $ratio  = $r / $maxint;
-      return ($max * $ratio) if ($r < $maxint);
+    # Divide first
+    if ($r < $maxint) {
+      $ratio = $r / $maxint;
+      $ratio *= $max;
+      return $ratio;
     }
   }
 }
 
+# Encapsulate the most common use case of wanting random integers in
+# the range [0,max]
+sub randint {
+  my ($self, $max) = @_;
+  return floor($self->rand($max + 1));
+}
+
 # The remaining subs are debugging purposes. They report back the
 # last random number in a variety of formats, but do not advance
 # to the next rand
 
 
 sub current {
-  return shift->{current};
+  return shift->[CURRENT];
 }
 
 sub as_string {			# alias for "current" method
-  return shift->{current};
+  return shift->[CURRENT];
 }
 
-# Unpacking as bytes or 32-bit unsigned ints. Use "network" order
-# for portability.
+# Unpacking as bytes or 32-bit unsigned ints. Using little-endian
+# since it's more common
 sub as_byte_array {
-  return unpack "C20", shift->{current};
+  return unpack "C20", shift->[CURRENT];
 }
 
 sub as_uint32_array {
-  return unpack "N5", shift->{current};
+  return unpack "V5", shift->[CURRENT];
 }
 
 
 sub as_hex {
-  return unpack "H40", shift->{current};
+  return unpack "H40", shift->[CURRENT];
 }
 
 # *nix-specific helper function to get a random 160-bit value from the
